// Sensor General
#define CHILD_ID_HUM 1
#define CHILD_ID_TEMP 2
#define CHILD_ID_BATT 3
#define SLEEP_TIME 300000

// define values for the battery measurement
#define R1 1.0e6
#define R2 470e3
#define VMIN 2.0
#define VMAX 2.6
#define ADC_PRECISION 1023
#define VREF 1.1

// Enable debug prints to serial monitor
#define MY_DEBUG 

// Mysensors Setup
#define MY_RADIO_NRF24
#define MY_RF24_PA_LEVEL RF24_PA_LOW
#define MY_BAUD_RATE 115200

// Node ID
#define MY_NODE_ID 02

/Singing
#define MY_SIGNING_ATSHA204
#define MY_SIGNING_ATSHA204_PIN 17
#define MY_SIGNING_REQUEST_SIGNATURES

//OTA
#define MY_OTA_FIRMWARE_FEATURE

#include <SPI.h>
#include <MySensors.h>
#include <Wire.h>
#include <Sodaq_SHT2x.h>

// Name of your Sensor
char SKETCH_NAME[] = "Example";
char SKETCH_VERSION[] = "1.0";

int BATTERY_SENSE_PIN = A0;  // select the input pin for the battery sense point

float lastTemp;
float lastHum;
boolean metric = true;
int batterie_check = 0;
int first_run = 1;
int send_batt = 1;
int force_send = 0;
float diff_temp = 0;
float diff_hum = 0;

MyMessage msgHum(CHILD_ID_HUM, V_HUM);
MyMessage msgTemp(CHILD_ID_TEMP, V_TEMP);
MyMessage msgVolt(CHILD_ID_BATT, V_VOLTAGE);


void setup() {

  // use the 1.1 V internal reference
  analogReference(INTERNAL);
  metric = getConfig().isMetric;
}

void presentation() {
  sendSketchInfo(SKETCH_NAME, SKETCH_VERSION);

  present(CHILD_ID_HUM, S_HUM);
  present(CHILD_ID_TEMP, S_TEMP);
  present(CHILD_ID_BATT, S_MULTIMETER);
}

void loop() {

float temperature = SHT2x.GetTemperature();

if (isnan(temperature)) {
    Serial.println("Failed reading temperature");
  } else  if (temperature >= lastTemp + 0.5 || temperature <= lastTemp - 0.5 || force_send == 6) {
    lastTemp = temperature;
    send(msgTemp.set(temperature, 1));
    Serial.print("Temperature: ");
    Serial.println(temperature);
    send_batt = 1;
  }

  float humidity = SHT2x.GetHumidity();
  if (isnan(humidity)) {
    Serial.println("Failed reading humidity");
  } else if (humidity >= lastHum + 2.0 || humidity <= lastHum - 2.0 || force_send == 6) {
    lastHum = humidity;
    send(msgHum.set(humidity, 1));
    Serial.print("Humidity: ");
    Serial.println(humidity);
    send_batt = 1;
  }

  // BATTERY MESSUREMENT ###########################################################################
  if (batterie_check >= 12 || first_run == 1 || send_batt == 1) {
    delay(50);
    int inputValue = analogRead(BATTERY_SENSE_PIN);

    float voltageDividerFactor = (R1 + R2) / R2;
    float maxValue = voltageDividerFactor * VREF;
    float voltsPerBit = maxValue / ADC_PRECISION;

    float batteryVoltage = voltsPerBit * inputValue;
    Serial.print("Battery Voltage: ");
    Serial.print(batteryVoltage);
    Serial.println(" V");
    send(msgVolt.set(batteryVoltage, 1));
    int batteryPercentage = ((batteryVoltage - VMIN) / (VMAX - VMIN)) * 100;
    Serial.print("Battery percent: ");
    Serial.print(batteryPercentage);
    Serial.println(" %");
    sendBatteryLevel(batteryPercentage);
    batterie_check = 1;
    first_run = 0;
    send_batt = 0;
  } else {
    batterie_check++;
  }
  // END BATT #####################################################################################

  if (force_send < 6) {
    force_send++;
  } else {
    force_send = 0;
  }

  sleep(SLEEP_TIME);

}


